// sort.cpp : Defines the entry point for the console application.
//

#include <cassert>
#include <limits>
#include <vector>
#include <ctime>
#include <string>
#include <iostream>
#include "achilles.h"

using namespace std;

typedef int TypeTested;
const TypeTested toFind = 1;

const double totalElementsSearched = 5000000000.;

int main(int argc, char** argv)
{
	if (argc < 3) 
	{
		cerr << "Wrong arguments.\n";
		return 1;
	}

	const bool useStd = argv[1][0] == 's';

	const unsigned int rangeSize = atoi(argv[2]);

	if (rangeSize == 0 || rangeSize >= totalElementsSearched) 
	{
		cerr << "Wrong arguments again.\n";
		return 1;
	}

	const unsigned int repeats = static_cast<unsigned int>(
		totalElementsSearched / rangeSize);

	cout << "Range size: " << rangeSize << ", repeats: " 
		<< repeats << endl;

	TypeTested* data = new TypeTested[rangeSize];
	TypeTested* dataEnd = data;
	for (; dataEnd != data + rangeSize; ++dataEnd)
	{
	    *dataEnd = TypeTested();
	}

	assert(toFind != TypeTested());

	dataEnd[-1] = toFind;

	cout << "Using " << (useStd ? "std" : "custom")
		<< " find routine ";

	const clock_t before = clock();
	unsigned int i = 0;

	if (useStd)
	{
    	for (; i != repeats; ++i)
    	{
		    // Time standard sort
		    find(data, dataEnd, dataEnd[-1]);
		}
	}
	else
	{
    	for (; i != repeats; ++i)
    	{
    		// Time custom sort
    		//Achilles::find(data, dataEnd, dataEnd[-1]);

			// The code below tests Duff's implementation
    		Achilles::find((const TypeTested*)data, 
				(const TypeTested*)dataEnd, dataEnd[-1]);
    	}
	}

	const unsigned int elapsed = 
		clock() - before;

	cout << "- elapsed seconds: " << double(elapsed) / CLOCKS_PER_SEC 
		<< endl;

	cout << "Time spent per element: " 
		<< (elapsed * 1000000000. / rangeSize / double(CLOCKS_PER_SEC)) / repeats
		<< " nanoseconds.\n";

	return 0;
}

